@@ -89,12 +89,13 @@ import errno
 import heapq
 import os
 import stat
+import sys
 import time
 import warnings
 
 __pychecker__ = 'no-reimportself'
 
-__version__ = '1.1'
+__version__ = '1.2'
 
 PERM_READ = 0400      # Read permission bit.
 PERM_WRITE = 0200     # Write permission bit.
@@ -132,6 +133,14 @@ class FakeLargeFileIoException(Error):
                    'fake large file: %s' % file_path)
 
 
+def CopyModule(old):
+  """Recompiles and creates new module object."""
+  saved = sys.modules.pop(old.__name__, None)
+  new = __import__(old.__name__)
+  sys.modules[old.__name__] = saved
+  return new
+
+
 class FakeFile(object):
   """Provides the appearance of a real file.
 
@@ -925,6 +934,7 @@ class FakePathModule(object):
   FakePathModule should *only* be instantiated by FakeOsModule.  See the
   FakeOsModule docstring for details.
   """
+  _OS_PATH_COPY = CopyModule(os.path)
 
   def __init__(self, filesystem, os_module=None):
     """Init.
@@ -934,44 +944,13 @@ class FakePathModule(object):
       os_module: (deprecated) FakeOsModule to assign to self.os
     """
     self.filesystem = filesystem
-    self._os_path = os.path
+    self._os_path = self._OS_PATH_COPY
     if os_module is None:
       warnings.warn(FAKE_PATH_MODULE_DEPRECATION, DeprecationWarning,
                     stacklevel=2)
-    self.os = os_module
+    self._os_path.os = self.os = os_module
     self.sep = self.filesystem.path_separator
 
-  def abspath(self, path):
-    """Determines a canonical absolute path for the argument file object.
-
-    Args:
-      path: Relative or absolute path to canonicalize.
-
-    Returns:
-      (str) A normalized, absolute path to the argument file.
-    """
-    path = self.normpath(path)
-    if path.startswith(self.sep):
-      return path
-    return self.normpath(self.join(self.filesystem.cwd, path))
-
-  def normpath(self, path):
-    return self.filesystem.CollapsePath(path)
-
-  def realpath(self, path):
-    """Returns canonical absolute path, eliminating symbolic links."""
-    if path is None:
-      # The real os.path.realpath(None) raises an AttributeError.
-      raise AttributeError('Invalid path: "%s"' % path)
-    # The real os.path.realpath('') returns the current working directory.
-    return self.filesystem.ResolvePath(path or self.filesystem.cwd)
-
-  def split(self, path):
-    return self.filesystem.SplitPath(path)
-
-  def join(self, *paths):
-    return self.filesystem.JoinPaths(*paths)
-
   def exists(self, path):
     """Determines whether the file object exists within the fake filesystem.
 
@@ -1039,12 +1018,6 @@ class FakePathModule(object):
     else:
       return path.startswith(self.filesystem.path_separator)
 
-  def basename(self, path):
-    return self.split(path)[1]
-
-  def dirname(self, path):
-    return self.split(path)[0]
-
   def isdir(self, path):
     """Determines if path identifies a directory."""
     return self._istype(path, stat.S_IFDIR)
@@ -1084,33 +1057,6 @@ class FakePathModule(object):
       raise OSError(errno.ENOENT, str(e))
     return file_obj.st_mtime
 
-  def walk(self, top, func, arg):
-    """Walks the directory tree starting at top.
-
-    As with the standard os.path.walk(), func() may alter the contents of fnames
-    in-place.
-
-    Args:
-      top:  root of the directory tree at which to begin the walk
-      func:  function to which to apply (arg, dirname, fnames) at each step in
-        the walk
-      arg:  argument to apply as first argument of func; may be used to collect
-        information about each step in the walk
-    """
-    try:
-      names = self.os.listdir(top)
-    except OSError, unused_error:
-      return
-    func(arg, top, names)
-    for name in names:
-      name = self.join(top, name)
-      try:
-        st = self.os.lstat(name)
-      except OSError, unused_error:
-        continue
-      if stat.S_ISDIR(st.st_mode):
-        self.walk(name, func, arg)
-
   def __getattr__(self, name):
     """Forwards any non-faked calls to os.path."""
     return self._os_path.__dict__[name]
@@ -1302,7 +1248,7 @@ class FakeOsModule(object):
     try:
       directory = self.filesystem.GetObject(target_directory)
     except IOError, (errno_code, msg):
-      raise OSError(errno_code, msg)
+      raise OSError(errno_code, msg, target_directory)
     if not directory.st_mode & stat.S_IFDIR:
       raise OSError(errno.ENOTDIR,
                     'Fake os module: not a directory',
@@ -1350,6 +1296,10 @@ class FakeOsModule(object):
     """Return current working directory."""
     return self.filesystem.cwd
 
+  def getcwdu(self):
+    """Return current working directory."""
+    return unicode(self.filesystem.cwd)
+
   def listdir(self, target_directory):
     """Returns a sorted list of filenames in target_directory.
 
@@ -1524,6 +1474,8 @@ class FakeOsModule(object):
       OSError:  if old_file does not exist.
       IOError:  if dirname(new_file) does not exist
     """
+    old_file = self.filesystem.NormalizePath(old_file)
+    new_file = self.filesystem.NormalizePath(new_file)
     if not self.filesystem.Exists(old_file):
       raise OSError(errno.ENOENT,
                     'Fake os object: can not rename nonexistent file '
@@ -1561,8 +1513,11 @@ class FakeOsModule(object):
 
     Raises:
       OSError: if target_directory does not exist or is not a directory,
-      or as per FakeFilesystem.RemoveObject.
+      or as per FakeFilesystem.RemoveObject. Cannot remove '.'.
     """
+    if target_directory == '.':
+      raise OSError(errno.EINVAL, 'Invalid argument: \'.\'')
+    target_directory = self.filesystem.NormalizePath(target_directory)
     if self._ConfirmDir(target_directory):
       if self.listdir(target_directory):
         raise OSError(errno.ENOTEMPTY, 'Fake Directory not empty',
@@ -1601,22 +1556,27 @@ class FakeOsModule(object):
         0777.  The umask is applied to this mode.
 
     Raises:
-      OSError: if the directory name is invalid,
+      OSError: if the directory name is invalid or parent directory is read only
       or as per FakeFilesystem.AddObject.
     """
     if dir_name.endswith(self.sep):
       dir_name = dir_name[:-1]
-    if self.filesystem.Exists(self.path.normpath(dir_name)):
-      raise OSError(errno.EEXIST, 'Fake object already exists', dir_name)
 
-    head, tail = self.path.split(dir_name)
-    if head:
-      base_dir = norm_head = self.path.normpath(head)
-      if head.endswith(self.sep + '..'):
-        base_dir, unused_dotdot, _ = head.partition(self.sep + '..')
+    parent_dir, _ = self.path.split(dir_name)
+    if parent_dir:
+      base_dir = self.path.normpath(parent_dir)
+      if parent_dir.endswith(self.sep + '..'):
+        base_dir, unused_dotdot, _ = parent_dir.partition(self.sep + '..')
       if not self.filesystem.Exists(base_dir):
         raise OSError(errno.ENOENT, 'No such fake directory', base_dir)
-      head = norm_head
+
+    dir_name = self.filesystem.NormalizePath(dir_name)
+    if self.filesystem.Exists(dir_name):
+      raise OSError(errno.EEXIST, 'Fake object already exists', dir_name)
+    head, tail = self.path.split(dir_name)
+    directory_object = self.filesystem.GetObject(head)
+    if not directory_object.st_mode & PERM_WRITE:
+      raise OSError(errno.EACCES, 'Permission Denied', dir_name)
 
     self.filesystem.AddObject(
         head, FakeDirectory(tail, mode & ~self.filesystem.umask))
@@ -1631,8 +1591,24 @@ class FakeOsModule(object):
         applied to this mode.
 
     Raises:
-      As per FakeFilesystem.CreateDirectory
+      OSError: if the directory already exists or as per
+      FakeFilesystem.CreateDirectory
     """
+    dir_name = self.filesystem.NormalizePath(dir_name)
+    path_components = self.filesystem.GetPathComponents(dir_name)
+
+    # Raise a permission denied error if the first existing directory is not
+    # writeable.
+    current_dir = self.filesystem.root
+    for component in path_components:
+      if component not in current_dir.contents:
+        if not current_dir.st_mode & PERM_WRITE:
+          raise OSError(errno.EACCES, 'Permission Denied', dir_name)
+        else:
+          break
+      else:
+        current_dir = current_dir.contents[component]
+
     self.filesystem.CreateDirectory(dir_name, mode & ~self.filesystem.umask)
 
   def access(self, path, mode):
